@@ -372,10 +372,28 @@ class QuoteMeOnThat(SoupTest):
         text = """<foo attr='bar "brawls" happen'>a</foo>"""
         soup = BeautifulSoup(text)        
         self.assertEquals(soup.renderContents(), text)
+
         soup.foo['attr'] = 'Brawls happen at "Bob\'s Bar"'
         newText = """<foo attr='Brawls happen at "Bob&squot;s Bar"'>a</foo>"""
         self.assertSoupEquals(soup.renderContents(), newText)
 
+        self.assertSoupEquals('<this is="really messed up & stuff">',
+                              '<this is="really messed up &amp; stuff"></this>')
+
+        # This is not what the original author had in mind, but it's
+        # a legitimate interpretation of what they wrote.
+        self.assertSoupEquals("""<a href="foo</a>, </a><a href="bar">baz</a>""",
+        '<a href="foo&lt;/a&gt;, &lt;/a&gt;&lt;a href="></a>, <a href="bar">baz</a>')
+
+        # SGMLParser generates bogus parse events when attribute values
+        # contain embedded brackets, but at least Beautiful Soup fixes
+        # it up a little.
+        self.assertSoupEquals('<a b="<a>">', '<a b="&lt;a&gt;"></a><a>"></a>')
+        self.assertSoupEquals('<a href="http://foo.com/<a> and blah and blah',
+                              """<a href='"http://foo.com/'></a><a> and blah and blah</a>""")
+
+        
+
 class YoureSoLiteral(SoupTest):
     "Test literal mode."
     def testLiteralMode(self):
@@ -533,6 +551,16 @@ class EncodeRed(SoupTest):
     """Tests encoding conversion, Unicode conversion, and Microsoft
     smart quote fixes."""
 
+    def testUnicodeDammitStandalone(self):
+        markup = "<foo>\x92</foo>"
+        dammit = UnicodeDammit(markup)
+        self.assertEquals(dammit.unicode, "<foo>&#x2019;</foo>")
+
+        hebrew = "\xed\xe5\xec\xf9"
+        dammit = UnicodeDammit(hebrew, ["iso-8859-8"])
+        self.assertEquals(dammit.unicode, u'\u05dd\u05d5\u05dc\u05e9')
+        self.assertEquals(dammit.originalEncoding, 'iso-8859-8')
+
     def testGarbageInGarbageOut(self):
         ascii = "<foo>a</foo>"
         unicodeData = u"<foo>\u00FC</foo>"
